****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_codedistribution.sas  
*
* Created (mm/dd/yyyy): 03/05/2018
* Last modified: 06/30/2019
* Version: 1.3
*
*--------------------------------------------------------------------------------------------------
* PURPOSE: Enumerate index defining codes
*   
*  Program inputs:                                                                                   
* 
*  Program outputs:                                                                                                                                       
* 
*  PARAMETERS:                                                                       
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1	 	  06/01/18	   AP		Added surveillance-related code
*
*	1.2	 	  12/18/18	   AP		Added PDX = X as valid code (when envelop is on)
*
*	1.3	 	  06/30/19	   AP		Removed type 3 censor output
*
***************************************************************************************************;

%macro ms_codedistribution();

	%put =====> MACRO CALLED: ms_codedistribution v1.3;

	%macro summarizecodedistribution(distindextype=);

		*EXP or HOI;
		%if "&distindextype" = "exp" %then %do;
			%let var = index;
			%let dindexdata = _groupindex(drop=group);

			%if %eval (&type. ne 4) %then %do;
				%let dindexdate = indexdt;
			%end;
			%else %do;
				%let dindexdate = EpisodeEndDt;
			%end;

		%end;
		%if "&distindextype" = "hoi" %then %do;
			%let var = fup;
			%let dindexdata = _fupevent;
			%let dindexdate = feventdt;
		%end;

		*Determine whether to produce table - skip if LABS or DATE define index no valid dates;
		data _lbparam;
			set _ITGroupParamLabs;
			where t&type._&var. = 'DEF';
		run;
		%isdata(dataset=_lbparam);
		%let LBPARAM = &nobs.;

		data _dtparam;
			set _ITGroupParamDate;
			where t&type._&var. = 'DEF';
		run;
		%isdata(dataset=_dtparam);
		%let DTPARAM = &nobs.;

		data _validdates;
			set _ptsmasterlist(where=(&dindexdate. is not missing) keep=&dindexdate.);
		run;
		%isdata(dataset=_validdates);
		%let VALIDDATES = &nobs.;

		%if %eval(&LBPARAM.=0) & %eval(&DTPARAM.=0) & %eval(&VALIDDATES.>0) %then %do;

			*Create list of index codes;
			*RX;
			proc sort data=_ITGroupParamNDC out=_rxparam(keep=group stockgroup) nodupkey;
				by group stockgroup t&type._index;
				where t&type._&var. = 'DEF';
			run;
			%isdata(dataset=_rxparam);
			%let RXPARAM = &nobs.;

			*PX/DX;
			proc sort data=_ITGroupParammeds out=_medsparam(keep=group codecat codetype enctype pdx code) nodupkey;
				by group t&type._index codecat codetype enctype pdx code;
				where t&type._&var. = 'DEF';
			run;
			%isdata(dataset=_medsparam);
			%let MEDPARAM = &nobs.;

			*MI;
			proc sort data=_ITGroupParamMIL out=_miparam(keep=group codecat codetype code) nodupkey;
				by group t&type._index codecat codetype code;
				where t&type._&var. = 'DEF';
			run;
			%isdata(dataset=_miparam);
			%let MIPARAM = &nobs.;

			%if %eval(&RXPARAM. >0) | %eval(&MEDPARAM. > 0) | %eval(&MIPARAM. > 0) %then %do;

				*expand EncType and PDX;
				data _medsparam_expanded_enc;
					set _medsparam;

					*All Caresetting and Principal;
					if EncType = "**" then do;
						do i = 1 to 5;
							enctype = scan('IP IS ED AV OA', i);
							output;
						end;
					end;
					else do;
						output;
					end;
				run;

				data _medsparam;
					set _medsparam_expanded_enc;

					if codecat = 'DX' & PDX = '*' then do;
						do p = 1 to 3;
							PDX = scan('P S X', p);
							output;
						end;
							PDX = '';
							output;
						drop p;
					end;
					else if codecat = 'PX' then do;
						%if %eval(&RUN_ENVELOPE. ne 2) %then %do;
						PDX = 'X';
						output;
						%end;
						PDX = '';
						output;
					end;
					else do;
						output;
					end;
				run;

				data _rxparamID
					 _medsparamID
					 _miparamID;
					set _rxparam (in=a)
						_medsparam (in=b)
						_miparam(in=c);
					distindexID = strip(put(_n_, 8.));

					if a then output _rxparamID;
					if b then output _medsparamID;
					if c then output _miparamID;
				run;

				*Restrict codes to index date;
				proc sql noprint;
				create table  _IndexCodes as 
					select in.*,
						   ref.group,
						   ref.&dindexdate.
					from &dindexdata. as in,
		                 _PtsMasterList as ref
				    where in.PatId=ref.PatId and 
		                  ref.&dindexdate. = in.Adate;  
				quit;

				*Merge ID;
				proc sql noprint;
					create table _indexcodesID as
					select distinct indx.patid,
									indx.group,
						 			indx.&dindexdate.,
						   			parm.distindexID
					from _IndexCodes as indx,
					_rxparamID as parm
					where indx.stockgroup = parm.stockgroup
				union corr all
					select distinct indx1.patid,
									indx1.group,
						 			indx1.&dindexdate.,
						   			parm1.distindexID
					from _IndexCodes as indx1,
					_medsparamID as parm1
					where indx1.codecat = parm1.codecat and 
						indx1.codetype = parm1.codetype and 
						indx1.enctype = parm1.enctype and 
						indx1.pdx = parm1.pdx and 
						indx1.code = parm1.code
				union corr all
					select distinct indx1.patid,
									indx1.group,
						 			indx1.&dindexdate.,
						   			parm1.distindexID
					from _IndexCodes as indx1,
					_miparamID as parm1
					where indx1.codecat = parm1.codecat and 
						indx1.codetype = parm1.codetype and
						indx1.code = parm1.code
				order by patid, &dindexdate., distindexID;
				quit;

				data _indexcodesID1;
					set _indexcodesID;
					by patid &dindexdate.;

					format distindexlist $250. distindextype $10.;
					distindextype = "&distindextype.";

					if first.&dindexdate. then do;
						distindexlist = distindexID;
					end;
					else do;
						distindexlist = catx('_',distindexlist,distindexID);
					end;
					retain distindexlist;

					ep=1;
					if last.&dindexdate. then output;
				run;

				*Summarize code counts;
				proc means data=_indexcodesID1 noprint nway;
					var ep;
					class group distindexlist distindextype;
					output out=_codecounts(drop=_:) sum= Episodes;
				run;

				proc sort data=_codecounts; 
					by descending episodes; 
				run;

				*Code to ID map;
				data _dindexmap;
					length group $40;
					format group $40. distindextype $10.;
					set _rxparamID _medsparamID _miparamID;
					group = "&itgroup.";
					distindextype = "&distindextype.";
					keep group distindextype stockgroup codecat codetype enctype pdx distindexID code;
				run;

				%if ("&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p") & %eval(&PERIODIDSTART.>1) %then %do;
					data _legacyvalues;
						set _PtsMasterList;
							where indexlook < &PERIODIDSTART.;
					run;

					*Add distindexlist to _mstr file for new patients;
					proc sql noprint undo_policy=none;
						create table _ptsmasterlist as
						select pts.*,
							 d.distindexlist as distindex&distindextype.
						from _ptsmasterlist(drop=distindex&distindextype. where=(&PERIODIDSTART. <= IndexLook  <= &PERIODIDEND.)) as pts
						left join _indexcodesID1 as d
						on pts.patid = d.patid and pts.&dindexdate. = d.&dindexdate.;
					quit;

					data _ptsmasterlist;
						set _ptsmasterlist
							_legacyvalues;
					run;
				%end;
				%else %do;
					*Add distindexlist to _mstr file;
					proc sql noprint undo_policy=none;
						create table _ptsmasterlist as
						select pts.*,
							 d.distindexlist as distindex&distindextype.
						from _ptsmasterlist as pts
						left join _indexcodesID1 as d
						on pts.patid = d.patid and pts.&dindexdate. = d.&dindexdate.;
					quit;
				%end;
			%end;
		%end;
		%else %do;
			data _codecounts;
				format group $40. distindextype $10. distindexlist $250.;
				group = "&itgroup.";
				distindextype = "&distindextype.";
				distindexlist = '';
				episodes = .;
			run;

			data _dindexmap;
				format group $40. distindextype $10. stockgroup $30. codecat $2. codetype $3. enctype $2. pdx $1. distindexid $8. code $11.;
				length group $40;
				group = "&itgroup.";
				distindextype = "&distindextype.";
				stockgroup = '';
				codecat = '';
				codetype = '';
				enctype = '';
				pdx = '';
				distindexid = '';
				code = '';
			run;

			data _ptsmasterlist ;
				set _ptsmasterlist;
				format distindex&distindextype. $250.;
				distindex&distindextype. = '';
			run;
		%end;

		*Clean-up;
		proc datasets nowarn noprint lib=work;
			delete _rxparam _medsparam _medsparam_expanded_enc _dtparam _lbparam _rxparamID _medsparamID _IndexCodes: _validdates;
		quit;
	%mend;

	%ISDATA(dataset=_PtsMasterList);
	%if %eval(&NOBS.>=1) %then %do;
		%summarizecodedistribution(distindextype=exp); 

		*Create group file;
		data _dindexcounts&group.;
			set _codecounts;
		run;
		data _dindexmap&group.;
			set _dindexmap;
		run;

		proc datasets nowarn noprint lib=work;
			delete _codecounts _dindexmap;
		quit;

		%if (%eval(&type.=2) | %eval(&type.=3)) & "&analysis" ne "tree" %then %do;
		%summarizecodedistribution(distindextype=hoi); 

			data _dindexcounts&group.;
				set _dindexcounts&group. _codecounts;
			run;
			data _dindexmap&group.;
				set _dindexmap&group. _dindexmap;
			run;

			proc datasets nowarn noprint lib=work;
				delete _codecounts _dindexmap;
			quit;
		%end;
	%end;
	%else %do;
		data _dindexcounts&group.;
			format group $40. distindextype $10. distindexlist $250.;
			group = "&itgroup.";
			distindextype = "exp";
			distindexlist = '';
			episodes = 0;
		run;

		data _dindexmap&group.;
			format group $40. distindextype $10. stockgroup $30. codecat $2. codetype $3. enctype $2. pdx $1. distindexid $8. code $11.;
			group = "&itgroup.";
			distindextype = "exp";
			stockgroup = '';
			codecat = '';
			codetype = '';
			enctype = '';
			pdx = '';
			distindexid = '';
			code = '';
		run;

		data _ptsmasterlist ;
			set _ptsmasterlist;
			format distindexexp $250.;
			distindexexp = '';

			%if %eval(&type.=2 | &type.=3) %then %do;
			format distindexhoi $250.;
			distindexhoi = '';
			%end;
		run;
	%end;

	*store to MSOC;
	%if %eval(&group.=1) %then %do;
		data msoc.&runid._distindex;
			retain group distindextype distindexlist episodes;
			set _dindexcounts&group.;
			keep group distindextype distindexlist episodes;
		run;

		data msoc.&runid._distindexmap;
			retain group distindextype stockgroup codecat codetype enctype pdx code distindexid;
			set _dindexmap&group.;
			keep group distindextype stockgroup codecat codetype enctype pdx code distindexid ;
		run;
	%end;
	%else %do;
 		proc append base=msoc.&runid._distindex data=_dindexcounts&group. force;
 		proc append base=msoc.&runid._distindexmap data=_dindexmap&group. force;
	%end;

	proc datasets nowarn noprint lib=work;
		delete _dindex:;
	quit;

	%put NOTE: ******** END OF MACRO: ms_codedistribution v1.3 ********;

%mend ms_codedistribution;
