****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createpov2.sas  
*
* Created (mm/dd/yyyy): 02/02/2017
* Last modified: 05/15/2018
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will identify all index dates with incidence claims in the washout period    
*
*  Program inputs:                                                                                   
*	-Dataset with potential index dates (POV1)  
*	-Dataset with incidence claims that should be outside the washout period
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with index dates that have an incidence claim in the washout period (POV2)
* 
* PARAMETERS:
*  	-CohortId = Identifier for the cohort being processed (primary, secondary)
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1       05/15/18	   AP		Added ExcludeSupply parameter
*
***************************************************************************************************;

%macro ms_createpov2(CohortId=);

%put =====> MACRO CALLED: ms_createpov2 v1.1;
	
	%let paramsuffix=;
	%if %str("&CohortId.") ne %str("") %then %do;
		%let paramsuffix=2;
	%end;

    proc sql noprint;
    create table _POV2&CohortId. as
    select distinct index.Patid,
                    index.Adate
    from _POV1&CohortId. as index, 
         _GroupWash&CohortId. as Inc
    where index.Patid = Inc.Patid and
		( ( inc.ExcludeSupply='N' and %MS_PeriodsOverlap(period1=index.Adate-min(Washper&paramsuffix.,99999) index.Adate-1,
                                       period2=Inc.ADate Inc.ExpireDt)) |
		    (inc.ExcludeSupply='Y' and %MS_PeriodsOverlap(period1=index.Adate-min(Washper&paramsuffix.,99999) index.Adate-1,
                                       period2=Inc.ADate)) )
    order by index.Patid, 
             index.Adate;
    quit;
	
%put NOTE: ********END OF MACRO: ms_createpov2 v1.1********;

%mend ms_createpov2;
