****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createt5episodes.sas  
*
* Created (mm/dd/yyyy): 03/11/2019
* Last modified: 
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will create episodes that occur after a selected index date.
*
*  Program inputs:                                                                                   
*	-Dataset with claims defined as being potential index dates 
*	-Dataset with valid index date 
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with all episodes
* 
* PARAMETERS:
*
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.1		  03/11/19	  AP		Removed drop statement for ITT that is no longer needed
*
***************************************************************************************************;

%macro ms_createt5episodes();

%put =====> MACRO CALLED: ms_createt5episodes v1.1;

	data _PtsMasterList;
		set _PtsMasterList;
		by patid indexdt;

		if T5CohortDef ='04' then do;
	           if first.PatId;
	       end;
	run;

	/*Select claims on or after index date*/
	data _groupindex2(drop=indexdt);
		length indexdt 8.;
	
		if _n_ = 1 then do;
			declare hash indx(dataset:'_PtsMasterList');
			indx.definekey('patid');
			indx.definedata('indexdt');
			call missing(indexdt);
			indx.definedone();
		end;
		
		set _GroupIndex;
		if indx.find()=0 then do;
			if adate >= indexdt then output;
		end;
	run;

	/*POV1*/
	%ms_createpov1(CohortId=2);

	/*POV4*/
	%DoITTCheck(CohortId=2);
	%ms_createpov4(CohortId=2);

	/*Create Episode list*/
	data _PtsMasterList2;
	format IndexDt EpisodeEndDt ENR_START ENR_END date9.;
       merge _POV12(in=pov1 rename=Adate=IndexDt)
             _POV42(in=pov4 rename=EpisodeStartDt=IndexDt)
             _IOT2(in=iot rename=EpisodeStartDt=IndexDt);
       by Patid IndexDt; * Adate represents potential Index Date;		
       
       if pov1 and pov4;

       *Index Date must overlap greater monitoring period;
       if &startdate. <= IndexDt <= &enddate.;

		OrigEpisEndDt = EpisodeEndDt;

		*Adjust EpisodeEndDt if EpisodeEndDt > enddate;
		%if &censor_qryend = 1 %then %do;
			if EpisodeEndDt > &enddate. then EpisodeEndDt = &enddate.;
		%end;

		/*Adjust EpisodeEndDt if EpisodeEndDt > DeathDt*/
		if EnrEpisodeCensoredAtDeath = 1 and EpisodeEndDt > DeathDt then EpisodeEnddt = deathdt;
	
        *Trucate episode at IOT;
		*Only adjust for IOT when IOT prior to EpisodeEndDate. IOT after EpisodeEndDt can 
		 occur if Epiosde shortened to occur on EndDate;
        if iot and TrunkDate and trunkdate <= EpisodeEndDt then EpisodeEndDt=TrunkDate;

    	*ExpExtPer may have made the episode to go beyond elig.;
    	if EpisodeEndDt > enr_end then EpisodeEndDt = enr_end;

		*Maximum Episode Duration;
        if not missing(maxepisdur) and EpisodeEndDt-IndexDt + 1 > maxepisdur then EpisodeEndDt = IndexDt + maxepisdur -1;  

    	*Defensive coding;
    	if IndexDt > EpisodeEndDt then delete;                

        keep PatID IndexDt EpisodeEndDt ENR_START ENR_END IndexLook RxSup RxAmt NumDispensing OrigEpisEndDt trunkdate EnrEpisodeCensoredAtDeath Deathdt maxepisdur IndexDtLookEndDt;
	run;

%put NOTE: ********END OF MACRO: ms_createt5episodes v1.1********;

%mend ms_createt5episodes;
