****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_evalsecondarytables.sas  
*
* Created (mm/dd/yyyy): 06/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:                                      
*   
*  Program inputs:  
*	-dplocal.&runid._mstr_multevent
*	-dplocal.&runid._mstr_overlap
*	-dplocal.&runid._mstr_epigap
*
*  Program outputs:    

*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_evalsecondarytables();

	%put =====> MACRO CALLED: ms_evalsecondarytables v1.0;

	*Select all output levels relevant to t2_multevent/t2_overlap;
	data _secepilevels
		 _t2epigaplevels;
		set userstrata;

		if lowcase(tableID) in ("t2multevent", "t2overlap") then output _secepilevels;

		if lowcase(tableID) = "t2epigap" then do;
			*defensive - add Epi_Gap if not specified since that is an automatic strata;
			if index(lowcase(levelvars), 'epi_gap') = 0 then do;
				levelvars = catx(' ',levelvars, "epi_gap");
			end;
			output _t2epigaplevels;
		end;
	run;

	%let t2secepistrat1 =;
	%let t2epigaplevels1 =;

	proc sql noprint;
		select distinct lowcase(levelvars) into: t2secepistrat1 separated by ' '
		from _secepilevels;
		select distinct lowcase(levelvars) into: t2epigaplevels1 separated by ' '
		from _t2epigaplevels;
	quit;

	%nonrep(invar = t2secepistrat1, outvar = t2secepistrat);
	%nonrep(invar = t2epigaplevels1, outvar = t2epigaplevels);

	/** Macro for calculating multiple events numerators **/
	%macro createmultieventnums(class=,level=);

		proc means data=_multmstr nway noprint missing;
	        var Patient AdjustedDisp RawDisp TotRxSup tte HadSecondaryEpisode NoSecondaryEpisode meets_adherence adherence_pat;
	        class analysisgrp PatId &class.;
	        output out=_num0(drop=_:) max(Patient)=npts           
	                                  sum(Patient)=Episodes    
	                                  sum(AdjustedDisp)=AdjustedCodeCount
	                                  sum(RawDisp)=RawCodeCount
	                                  sum(TotRxSup)=DaySupp
									  sum(tte) = tte
	    						      sum(HadSecondaryEpisode)=Eps_wSecEpi
	 								  sum(NoSecondaryEpisode)=Eps_woSecEpi
	                                  sum(meets_adherence)=Adhere_Episodes
	 								  max(adherence_pat)=Adhere_npts;         
	        run;

		proc means data=_num0 missing nway noprint;
	        var npts Episodes AdjustedCodeCount RawCodeCount DaySupp tte Eps_wSecEpi Eps_woSecEpi Adhere_Episodes Adhere_npts ;
	        class analysisgrp &class.;
	        output out=_outlvl(drop=_:) sum=;             
		run;

	   	%if %eval(&s.=1) %then %do;
			data _numcounts;
		    	set _outlvl;
		        format level $3.;
		        Level="&level.";
		    run;
		%end;
		%else %do;
			data _numcounts;
		   		set _numcounts
		            _outlvl(in=a);
		        if a then do;	
					level="&level.";
				end;
			run;
	    %end;

		proc datasets nowarn noprint lib=work;
			delete _outlvl _num0;
		quit;

	%mend;


	/** Macro for calculating episode gap **/
	%macro outputepigap(class=, level=);

		proc means data=_epigapmstr nway noprint missing;
			var count1 count2 count3;
		    class analysisgrp &class. / missing;
			output out=_outepigap(drop=_:) sum=;         
		run;

	   	%if %eval(&s.=1) %then %do;
			data _numepicounts;
		    	set _outepigap;
		        format level $3.;
		        Level="&level.";
		    run;
		%end;
		%else %do;
			data _numepicounts;
		   		set _numepicounts
		            _outepigap(in=a);
		        if a then do;	
					level="&level.";
				end;
			run;
	    %end;

		proc datasets nowarn noprint lib=work;
			delete _outepigap;
		quit;

	%mend;


	/** Macro for calculating overlap numerators **/
	%macro createoverlapnums(class=,level=);

	   	proc means data=_overlapmstr nway noprint missing;
	   		var Patient AdjustedDisp1 AdjustedDisp2 RawDisp1 RawDisp2 TotRxSup1 TotRxSup2 HadSecondaryEpisode NoSecondaryEpisode
				meets_adherence adherence_pat tte total_days_overlap 
				%do b = 1 %to &num_cutoffcat.;
					%scan(&cutoff_varlist., &b., ' ')
				%end; ;
	      	class analysisgrp PatId &class.;
	       	output out=_num0(drop=_:) max(Patient)=npts           
	                              	  sum(Patient)=Episodes    
	                               	  sum(AdjustedDisp1)=AdjustedCodeCount1
	                               	  sum(AdjustedDisp2)=AdjustedCodeCount2
	                               	  sum(RawDisp1)=RawCodeCount1
	                               	  sum(RawDisp2)=RawCodeCount2
	                               	  sum(TotRxSup1)=DaySupp1
								  	  sum(TotRxSup2)=DaySupp2
	    					      	  sum(HadSecondaryEpisode)=Eps_wSecEpi
	    					      	  max(HadSecondaryEpisode)=npts_wSecEpi
	 							  	  sum(NoSecondaryEpisode)=Eps_woSecEpi
								  	  max(NoSecondaryEpisode)=npts_woSecEpi
	                               	  sum(meets_adherence)=ADHERE_Episodes
	 							  	  max(adherence_pat)=ADHERE_npts       
 								  	  sum(tte) = tte
								  	  sum(total_days_overlap) = total_overlap
									  %do b = 1 %to &num_cutoffcat.;
									  	sum(%scan(&cutoff_varlist., &b., ' ')) = epi%scan(&cutoff_varlist., &b., ' ')
									    max(%scan(&cutoff_varlist., &b., ' ')) = npts%scan(&cutoff_varlist., &b., ' ')
									  %end;
									  ;
		run;

	    proc means data=_num0 missing nway noprint;
			var npts Episodes AdjustedCodeCount1 AdjustedCodeCount2 RawCodeCount1 RawCodeCount2 DaySupp1 DaySupp2 Eps_wSecEpi npts_wSecEpi Eps_woSecEpi npts_woSecEpi
					ADHERE_Episodes ADHERE_npts tte total_overlap
	 					%do b = 1 %to &num_cutoffcat.;
							epi%scan(&cutoff_varlist., &b., ' ') npts%scan(&cutoff_varlist., &b., ' ')
						%end;;
		    class analysisgrp &class.;
		    output out=_outlvl(drop=_:) sum=;             
		run;

		%if %eval(&s.=1) %then %do;
			data _numcounts;
		    	set _outlvl;
		        format level $3.;
		        Level="&level.";
		    run;
		%end;
		%else %do;
			data _numcounts;
		   		set _numcounts
		            _outlvl(in=a);
		        if a then do;	
					level="&level.";
				end;
			run;
		%end;

		proc datasets nowarn noprint lib=work;
			delete _outlvl _num0;
		quit;
	      
	%mend;

	/****************************************************************/
	/** Loop through Analysis Groups, and calculate output metrics **/
	/****************************************************************/

	%ISDATA(dataset=secondaryinputfile);
	%let analysisgroup_n = &nobs.;
	%do group=1 %to &analysisgroup_n.;

		*Select primary group for the analysis;
		data _null_;
			set secondaryinputfile;
				if _n_ = &group. then do;
				call symputx("primarygrp", primary);
				call symputx("analysisgrp", analysisgrp);

				%if %eval(&overlap.=1) %then %do;
					call symputx("cutoffcat", cutoffcat);
					call symputx("cutoffcat_scale", lowcase(cutoffcat_scale));
				%end;
			end;
		run;
				
		data _null_;
			set infolder.&COHORTFILE.;
			where upcase(cohortgrp) = upcase("&primarygrp.");
				*Age stratification;
				if missing(agestrat) then do;
					call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
				end;
				else do;
					call symputx("agestrat",agestrat);
				end;
		run;

		%put age is &agestrat;

		%let cutoff_varlist = ;
		%let num_cutoffcat = ;
		%let cutoff_boolean = ;
		%macro wrapper();

			/*Calculate cut-off variables*/
			%if "&cutoffcat_scale" = "percent" %then %do;
				%let var = percent_overlap;
			%end;
			%else %do;
				%let var = total_days_overlap;
			%end;

			%let temp_cutoffcat = ;
			%let temp_cutoffcat = %sysfunc(tranwrd(&cutoffcat., -, _));
			%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., >, gt));
			%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., <, lt));
			%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., =, e));
			%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., lte, le));
			%let temp_cutoffcat = %sysfunc(tranwrd(&temp_cutoffcat., gte, ge));
							
			%let num_cutoffcat = %sysfunc(countw(&cutoffcat, ' '));
			%do loop = 1 %to &num_cutoffcat;
				/*variable list*/
				%let cutoff_varlist = &cutoff_varlist. %sysfunc(cats(_,%scan(&temp_cutoffcat., &loop.)));
				/*mathematical expressions*/
				%let cat = %scan(&cutoffcat., &loop., ' ');
				%put &cat.;
				/*no range of values (i.e. <10, <=20, >60 >=75)*/
				%if %index(&cat., -) = 0 %then %do;
					%let cat = &var.&cat.;
				%end;
				/*range of values inclusive of boundry (i.e. 25-50)*/
				%else %if %index(&cat., -) > 0 & %index(&cat., <) = 0 & %index(&cat., >) = 0 %then %do;
					%let cat = %sysfunc(tranwrd(&cat., -, <=&var.<=));
				%end;
				/*range of values inclusive of upper boundry, (i.e. 25<-60)*/
				%else %if %index(&cat., -) > 0 & %index(%scan(&cat., 1, '-'), <) > 0 %then %do;
					%let cat = %scan(&cat., 1, '-')&var.<=%scan(&cat., 2, '-');
				%end;
				/*range of values inclusive of lower bountry, (i.e. 50-<75)*/
				%else %if %index(&cat., -) > 0 & %index(%scan(&cat., 2, '-'), <) > 0 %then %do;
					%let cat = %scan(&cat., 1, '-')<=&var.%scan(&cat., 2, '-');
				%end;
				%let cutoff_boolean = &cutoff_boolean. &cat.;
				%end;
			%put &cutoffcat &cutoff_varlist. &cutoff_boolean.;
		%mend;
		%if %eval(&overlap.=1) %then %do;
		%wrapper();
		%end;

		*reset macro vars;
		%let commonlevellist = ;
		%let uncommonlevellist = ;

		%isdata(dataset=_secepilevels);   
		%if %eval(&nobs.>0) %then %do; 
			%do s = 1 %to %eval(&nobs.);
				data _null_;
					set _secepilevels;
					if _n_ = &s. then do;
						call symputx('levelid', levelid);
						call symputx('levelvars', levelvars);
					end;
				run;

				%if %eval(&multievent.=1) %then %do;

					*Square stratifications vars;
					%ms_squaredtableshell(squarevars =&levelvars.,
								  		  analysisvars=adjusteddisp rawdisp totrxsup adherence_pat tte,
								  		  groups="&analysisgrp.",
										  analysis=Y);

					*Merge squared values with _mstr_multevent;
					data _multmstr;
					    set dplocal.&runid._mstr_multevent(in=a where=(analysisgrp="&analysisgrp."))
					        _mastersquare(in=b);
					    	if a then do;
					        	month=month(indexdt);
					        	patient=1;
					        	if epi_count>0 then do;
									HadSecondaryEpisode=1;
									NoSecondaryEpisode=0;
								end;
								else do;
									HadSecondaryEpisode=0; 
									NoSecondaryEpisode=1;
								end;	 
								if adherence = 1 then meets_adherence = 1; /*re-code to new var b/c also being used as stratification var*/
								else meets_adherence =0;
							end;
					    	else do;
					        	patient=0;    
					       		HadSecondaryEpisode=0; 
								NoSecondaryEpisode = 0;
								meets_adherence = 0; 
								adherence_pat=0;
								epi_count=0;
					    	end;
					run;

					/*Summarize data */
				  	%createmultieventnums(class=&levelvars.,level=&levelid.);

					proc datasets nowarn noprint lib=work;
						delete _mastersquare _multmstr;
					quit;

				%end; /*multievent*/

				%if %eval(&overlap.=1) %then %do;

					*Square stratifications vars;
					%ms_squaredtableshell(squarevars =&levelvars.,
								  		  analysisvars=	adjusteddisp1 adjusteddisp2 rawdisp1 rawdisp2 totrxsup1 totrxsup2
														%do b = 1 %to &num_cutoffcat.;
															%scan(&cutoff_varlist., &b., ' ') 
														%end;,
								  		  groups="&analysisgrp.",
										  analysis=Y);

					*Merge Squared data with _mstr_overlap;
					data _overlapmstr;
					    set dplocal.&runid._mstr_overlap(in=a where=(analysisgrp="&analysisgrp."))
					        _mastersquare(in=b);
					    	if a then do;
					        	month=month(indexdt);
					        	patient=1;
					        	if epi_count>0 then do;
									HadSecondaryEpisode=1;
									NoSecondaryEpisode=0;
								end;
								else do;
									HadSecondaryEpisode=0; 
									NoSecondaryEpisode=1;
								end;	 
								if adherence = 1 then meets_adherence = 1; /*re-code to new var b/c also being used as stratification var*/
								else meets_adherence =0;
							end;
					    	else do;
					        	patient=0;    
					       		HadSecondaryEpisode=0; 
								NoSecondaryEpisode = 0;
								meets_adherence = 0; 
								adherence_pat=0;
								tte = 0;
								total_days_overlap = 0;
					    	end;
					run;

					/*Summarize data */
					%createoverlapnums(class=&levelvars.,level=&levelid.);

					proc datasets nowarn noprint lib=work;
						delete _mastersquare _multmstr;
					quit;

				%end; /*overlap*/
			%end; /*loop through stratifications*/

			*The table to be generated in MSOC will be the result of a merge between numerators and donominators;
			data _numcounts;
				set _numcounts;
					*initialize missing vars to prevent merge w a r n i n g s;
					length race hispanic zip_uncertain $1. zip3 state hhs_reg cb_reg $7.;
						if _n_ = 1 then do;
						dsid = open("_NumCounts");
							if varnum(dsid,"race") = 0 then race ='';
							if varnum(dsid,"hispanic") = 0 then hispanic ='';
							if varnum(dsid,"zip_uncertain") = 0 then zip_uncertain ='';
							if varnum(dsid,"zip3") = 0 then zip3 ='';
							if varnum(dsid,"state") = 0 then state ='';
							if varnum(dsid,"hhs_reg") = 0 then hhs_reg ='';
							if varnum(dsid,"cb_reg") = 0 then cb_reg ='';

							if varnum(dsid,"sex") = 0 then sex ='';
							if varnum(dsid,"agegroup") = 0 then agegroup ='';
							if varnum(dsid,"agegroupnum") = 0 then agegroupnum =.;
							if varnum(dsid,"year") = 0 then year =.;
							if varnum(dsid,"month") = 0 then month =.;
						rc= close(dsid);
					end;
					drop rc dsid;
			run;

			proc sort data=_numcounts;
				by analysisgrp level sex AgeGroupNum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic  ;
			run; 

			*Denominators are for primary group;
			proc sort data=dplocal.&runid._denomcounts out=_denomcounts;
				by group level sex AgeGroupNum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic ;
				where upcase(group) = upcase("&primarygrp.");
			run; 

			*Add Analysisgrp;
			data _denomcounts;
				format analysisgrp $40.;
				set _denomcounts;
				analysisgrp = "&analysisgrp.";
				drop group;
			run;

			*Common level values between Nums and Denoms - merge these then merge in Num only data;
			proc sql noprint;
				select distinct quote(trim(left(num.level))) into: commonlevellist separated by ' '
				from _numcounts as num,
					 _denomcounts as den
				where num.level= den.level;

				select distinct quote(trim(left(num.level))) into: uncommonlevellist separated by ' '
				from _numcounts as num
			%if %str(&commonlevellist.) ne %str() %then %do;
				where num.level not in (&commonlevellist)
				%end;
				;
			quit;
			%put levels with denoms: &commonlevellist.;
			%put levels without denoms: &uncommonlevellist.;

			%if %str(&commonlevellist.) ne %str() %then %do;
			data _commonlevels;
				merge _NumCounts (in=a)
					  _DenomCounts (in=b);
				by analysisgrp Level sex AgeGroupNum year month zip3 state hhs_reg cb_reg zip_uncertain race hispanic ;
					where level in (&commonlevellist);

				if not b then do;
					DenNumPts=0;
					DenNumMemDays=0;
				end;

				if b and missing(DenNumPts) then do;
					DenNumPts=0;
					DenNumMemDays=0;
				end;
			run;
			%end;

			data _combined;
				length race hispanic zip_uncertain $1. zip3 state hhs_reg cb_reg $7.;
				set 
					%if %str(&commonlevellist.) ne %str() %then %do;
					_commonlevels (in=a)
					%end;
					%if %str(&uncommonlevellist.) ne %str() %then %do;
					_numcounts (in=b where=(level in (&uncommonlevellist.)))
					%end;
					;
							
				%if %str(&uncommonlevellist.) ne %str() %then %do;
				if b then do;
					DenNumPts=.;
					DenNumMemDays=.;
				end;
				%end;
			run;

			*Save to DPLOCAL/MSOC;
			%if %eval(&group.)=1 %then %do;
				data DPLOCAL.&RUNID._numcounts;
					set _numcounts;
				run;

				%if %eval(&multievent.=1) %then %do;
				data MSOC.&RUNID._t2_multevent;
					retain analysisgrp Level &t2secepistrat. 
						Npts Episodes AdjustedCodeCount RawCodeCount DaySupp tte eps_wsecepi eps_wosecepi adhere_episodes adhere_npts dennumpts dennummemdays;
					set _combined;
					keep analysisgrp Level &t2secepistrat. 
						Npts Episodes AdjustedCodeCount RawCodeCount DaySupp tte eps_wsecepi eps_wosecepi adhere_episodes adhere_npts dennumpts dennummemdays;
				run;			
				%end;

				%if %eval(&overlap.=1) %then %do;
				data MSOC.&RUNID._t2_overlap;
					retain analysisgrp Level &t2secepistrat. 
						Npts Episodes AdjustedCodeCount1 AdjustedCodeCount2 RawCodeCount1 RawCodeCount2 DaySupp1 DaySupp2 Eps_wSecEpi npts_wSecEpi Eps_woSecEpi npts_woSecEpi
							tte total_overlap adhere_episodes adhere_npts   
								%do b = 1 %to &num_cutoffcat.;
									epi%scan(&cutoff_varlist., &b., ' ')
									npts%scan(&cutoff_varlist., &b., ' ')
								%end; 
						dennumpts dennummemdays;
					set _combined;
					keep analysisgrp Level &t2secepistrat. 
						Npts Episodes AdjustedCodeCount1 AdjustedCodeCount2 RawCodeCount1 RawCodeCount2 DaySupp1 DaySupp2 Eps_wSecEpi npts_wSecEpi Eps_woSecEpi npts_woSecEpi
							tte total_overlap adhere_episodes adhere_npts dennumpts dennummemdays  
								%do b = 1 %to &num_cutoffcat.;
									epi%scan(&cutoff_varlist., &b., ' ')
									npts%scan(&cutoff_varlist., &b., ' ')
								%end; 
						;
				run;
				%end;

			%end;
			%else %do;
				proc append base=DPLOCAL.&RUNID._numcounts
				            data=_numcounts force;
				run;

				%if %eval(&multievent.=1) %then %do;
				proc append base=MSOC.&RUNID._t2_multevent
				            data=_combined(keep=analysisgrp Level &t2secepistrat. Npts Episodes AdjustedCodeCount RawCodeCount DaySupp 
												tte eps_wsecepi eps_wosecepi adhere_episodes adhere_npts dennumpts dennummemdays) force;
			   	run;
				%end;

				%if %eval(&overlap.=1) %then %do;
				proc append base=MSOC.&RUNID._t2_overlap
				            data=_combined(keep= analysisgrp Level &t2secepistrat. 
						Npts Episodes AdjustedCodeCount1 AdjustedCodeCount2 RawCodeCount1 RawCodeCount2 DaySupp1 DaySupp2 Eps_wSecEpi npts_wSecEpi Eps_woSecEpi npts_woSecEpi
							tte total_overlap adhere_episodes adhere_npts dennumpts dennummemdays
								%do b = 1 %to &num_cutoffcat.;
									epi%scan(&cutoff_varlist., &b., ' ')
									npts%scan(&cutoff_varlist., &b., ' ')
								%end; ) force;
			    run;
				%end;

			%end;
		%end; /*_multievent/_overlap output table*/

		/****************/
		/* EpiGap Table */
		/****************/
		%if %eval(&multievent.=1) %then %do;
			%isdata(dataset=_t2epigaplevels);
			%if %eval(&nobs.>0) %then %do;
				%do s = 1 %to %eval(&nobs.);
					data _null_;
						set _t2epigaplevels;
						if _n_ = &s. then do;
							call symputx('levelid', levelid);
							call symputx('levelvars', levelvars);
						end;
					run;

					%ms_squaredtableshell(squarevars =&levelvars.,
								  		  analysisvars=,
								  		  groups="&analysisgrp.",
										  analysis=Y);
			
					data _epigapmstr;
					    set dplocal.&runid._mstr_epigap(in=a where=(analysisgrp="&analysisgrp."))
					        _mastersquare(in=b);
					    	if a then do;
					        	month=month(indexdt);
							end;
					    	else do;
					        	count1=0;    
					       		count2=0; 
								count3=0;
					    	end;
				    run;

					%outputepigap(class=&levelvars.,level=&levelid.);

					proc datasets nowarn noprint lib=work;
						delete _mastersquare _epigapmstr;
					quit;
				%end;

				*Save to MSOC;
				%if %eval(&group.=1) %then %do;
					data MSOC.&RUNID._t2_epigap;
						retain analysisgrp Level &t2epigaplevels. count1 count2 count3;
						set _numepicounts;
						keep analysisgrp Level &t2epigaplevels. count1 count2 count3;
					run;
				%end;
				%else %do;
					proc append base=MSOC.&RUNID._t2_epigap
				         data=_numepicounts(keep=analysisgrp Level &t2epigaplevels. count1 count2 count3) force;
			    	run;
				%end;
			%end;
		%end;

	%end; /*loop through groups*/

	proc datasets nowarn noprint lib=work;
		delete _combined _epigap:;
	quit;

%put NOTE: ********END OF MACRO: ms_evalsecondarytables v1.0********;

%mend ms_evalsecondarytables;
