****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_time.sas 
*
* Created (mm/dd/yyyy): 07/31/2014
* Last modified: 06/30/2019
* Version: 1.2
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  This program is composed of two macros: ms_starttimer and ms_stoptimer;
*
*  Program inputs:                                                                                   
*                                     
*  Program outputs: 
*
* PARAMETERS:
*
*  ms_starttimer(timestart=)
*      timestart=Name of the macro variable that will hold the time.
                 Two additional macro variables created to hold date and time parts
*      Example: %ms_starttimer(timestart=MAINSTART)
*  
*  ms_stoptimer(timestart=)
*      timestart=Reference to the name of the macro variable that will hold the time.
*      Example: %ms_stoptimer(timestart=MAINSTART)
*
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1      02/06/15   SL(EM)     Put statement was modified in ms_starttimer to display start 
*                                      date and time in the log.
*	 
*	 1.2	  06/30/19   AP			Removed put statements to make macro re-callable
*
***************************************************************************************************;

/*******************************************************************/
/*  ms_starttimer 							  					   */
/*******************************************************************/
%macro ms_starttimer(timestart=);

	%put =====> MACRO CALLED: ms_time v1.2 => ms_starttimer;

	%global &timestart. &timestart.date &timestart.time;

	data _NULL_;
	   temp=DATETIME();
	   call symputx("&timestart.",temp);
	   call symputx("&timestart.date",put(datepart(temp),date9.));
	   call symputx("&timestart.time",put(timepart(temp),hhmm8.2));
	run;

	%put NOTE: ********END OF MACRO: ms_time v1.2 => ms_starttimer ********;

%mend;


/*******************************************************************/
/*  ms_stoptimer 							  					   */
/*******************************************************************/
%macro ms_stoptimer(timestart=);

	%put =====> MACRO CALLED: ms_time v1.2 => ms_stoptimer;

	%put "&&&timestart.";

	%global STOP hours minutes seconds;
	%global totalseconds &timestart.enddate &timestart.endtime;

	data _NULL_;
	   temp=DATETIME();
	   seconds=temp-&&&timestart.;
	   hours=int(seconds/3600);
	   minutes=int((seconds-hours*3600)/60);
	   seconds2=int((seconds-hours*3600-minutes*60));
	   call symput("STOP",put(temp,best.));
	   call symput("hours",put(hours,4.0));
	   call symput("minutes",put(minutes,2.0));
	   call symput("seconds",put(seconds2,2.0));

	   call symputx("&timestart.enddate",put(datepart(temp),date9.));
	   call symputx("&timestart.endtime",put(timepart(temp),hhmm8.2));
	run;

	%put NOTE: ********END OF MACRO: ms_time v1.2 => ms_stoptimer ********;

%mend;
